/*
 * serial.c
 *
 * Created: 2015-04-09 14:16:34
 *  Author: dsosnowski
 */ 


#define F_CPU 12000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"

#define CORE_CLOCK		8000000
#define BAUD_RATE		9600

#define SERIAL_PORT		UART_PORT
#define SERIAL_DDR		UART_DDR
#define SERIAL_RXD		UART_RXD
//#define SERIAL_TXD		UART_TXD

#define SERIAL_ENABLE	UCSRB = (1<<RXEN)		
//#define SERIAL_ENABLE	UCSRB = (1<<RXEN | 1<<TXEN)	

//$$GPRMC,150923.00,A,5221.59521,N,02102.09627,E,0.449,,100415,,,A*73$0D

//-------------------------------------------
struct {
	char rbuff[SERIAL_RBUFFER_SIZE];
	uint16_t rcounter;
	char *rpt;
	volatile char status;
}serial;

//-------------------------------------------
struct {
	char status;
	char seconds;
	char minutes;
	char hours;
	char dayofmonth;
	char month;
	char year;
	char baud;
	char frame[6];
} gps;

const char baudtable[] = {207, 103, 68, 51};	//0-4800, 1-9600, 2-14400, 3-19200

//-------------------------------------------
void SerialInit(void)
{
	if (gps.baud > 3) gps.baud = 3;
	//gps.baud = 3;
	
	SERIAL_PORT |= (1<<SERIAL_RXD);	//config ports
	//SERIAL_DDR |= (1<<SERIAL_TXD);
	 	
	//config USART
	UCSRA = 0;
	SERIAL_ENABLE;
	UCSRC = (1<<UCSZ1 | 1<<UCSZ0);				//8 data, 1 stop, parity none
	UBRRH = 0;
	//UBRRL = 51;									//UBRRL = 51 - 9600
	UBRRL = baudtable[gps.baud];				//UBRRL = 51 - 9600
	
	
	gps.frame[0] = '$';
	gps.frame[1] = 'G';
	gps.frame[2] = 'P';
	gps.frame[3] = 'R';
	gps.frame[4] = 'M';
	gps.frame[5] = 'C';	
}

//----------------------------------------------------
void SerialEnable(char action)
{
	if (action == 1)
	{
		serial.status = SERIAL_IDLE;
		SERIAL_ENABLE | (1<<RXCIE);
	}
	else
	{
		SERIAL_ENABLE;
	}
}

//----------------------------------------------------
char SerialStatus(void)
{
	return serial.status;
}

//----------------------------------------------------
char *SerialClockGet(void)
{
	return &gps.seconds;
}

//----------------------------------------------------
char *SerialConfigGet(void)
{
	return &gps.baud;
}

//----------------------------------------------------
char SerialClock(void)
{
	int16_t i;
	
	if (serial.status == SERIAL_READY)
	{
		gps.status = 1;
		if (gps.status == 1)
		{
			for(i = 0; i < 6; i++) 
			{
				if (serial.rbuff[i] != gps.frame[i]) gps.status = 2;
			}
		}
		
		//find time pos
		if (gps.status == 1)
		{
			i = SerialFindChar(',', 1);
			if (i > 0) 
			{
				if (SerialCheckValue(i, 6) == 1)
				{
					gps.hours = SerialGetValue(i);
					gps.minutes = SerialGetValue(i+2);
					gps.seconds = SerialGetValue(i+4);
				}
				else
					gps.status = 4;
			}
			else
				gps.status = 3;
		}

		//find date pos
		if (gps.status == 1)
		{
		i = SerialFindChar(',', 9);
		if (i > 0)
		{
			if (SerialCheckValue(i, 6) == 1)
			{
				gps.dayofmonth = (SerialGetValue(i) - 1); 
				gps.month = (SerialGetValue(i+2) - 1);
				gps.year = SerialGetValue(i+4);
			}
			else
				gps.status = 6;
		}
		else
			gps.status = 5;		
		}
	}
	else
		gps.status = 0;
	
	//SerialEnable(1);

	return gps.status;
}

//---------------------------------------------------
int16_t SerialFindChar(char chr, char pos)
{
	int16_t i = 0;
	char c = 0;
	char status = 0;
	
	while (status == 0)	
	{
		if (serial.rbuff[i] == chr) c++;
		if (c >= pos) status = 1;
		i++;
		if (i >=  SERIAL_RBUFFER_SIZE) status = 2;
	}
	if (status == 1) return i;
	return 0;
}

//---------------------------------------------------
char SerialCheckValue(char pos, char len)
{
	int16_t i = pos;
	char c = 0;
	char status = 0;
	
	while (status == 0)
	{
		if ((serial.rbuff[i] < '0') || (serial.rbuff[i] > '9')) status = 2;
		c++;
		if ((c >= len) && (status == 0)) status = 1;
		
		i++;
		if (i >=  SERIAL_RBUFFER_SIZE) status = 2;
	}
	if (status == 1) return 1;
	return 0;
}

//---------------------------------------------------
char SerialGetValue(char pos)
{
	int16_t i = pos;
	char temp = 0;
	
	temp += ((serial.rbuff[i] - '0') * 10);
	i++;
	temp += (serial.rbuff[i] - '0');
	return temp;
}

//----------------------------------------------------
ISR(USART_RXC_vect)
{
	char temp = UDR;						//get byte from data register
	
	//----------------------------------
	if (serial.status == SERIAL_IDLE)
	{
		if (temp == COM_PREFIX)
		{
			serial.rcounter = 0;
			serial.rpt = &serial.rbuff[0];
			serial.status = SERIAL_READING;
		}
	}

	//----------------------------------
	if (serial.status == SERIAL_READING)
	{
		if ((serial.rcounter == 3) && (temp != 'R')) serial.status = SERIAL_IDLE;
		if ((serial.rcounter == 4) && (temp != 'M')) serial.status = SERIAL_IDLE;
		if ((serial.rcounter == 5) && (temp != 'C')) serial.status = SERIAL_IDLE;

		//----------------------------------
		if (serial.rcounter < SERIAL_RBUFFER_SIZE)
		{
			*serial.rpt = temp;
			serial.rpt++;
			*serial.rpt = 0;
			serial.rcounter++;
		}
		else
			 serial.status = SERIAL_IDLE;
	
		//----------------------------------
		if (temp == COM_POSTFIX)
		{
			serial.status = SERIAL_READY;		
			SERIAL_ENABLE;			//RX disable	
		}
	}
}

//-------------------------------------
void SerialSend(char data)
{
	//while(!(UCSRA & 1<<UDRE));
	//UDR = data;
}

//---------------------------------
void SerialText(char *str)
{
	while(*str && str)
	SerialSend(*str++);
}

//---------------------------------
void SerialMsg(char *str)
{
	SerialSend('\r');
	SerialSend('\n');
	SerialText(str);
}

//-----------------------------------
void SerialSimpleVal(char val)
{
	if (val > 99) val = 99;
	 SerialSend((val / 10) + '0');
	 SerialSend((val % 10) + '0');
}

//-----------------------------------
void SerialSignedVal(int16_t val)
{
	//unsigned int value;
	unsigned int temp = 1000;
	char digit;
	char blank = 1;
	if (val & 0x8000)
	{
		SerialSend('-');
		val = ((~val) + 1);
	}
	//else
	//SerialSend(' ');

	if (val > 9999) val = 9999;
	while (temp)
	{
		digit = val / temp;
		val = val % temp;
		
		if (temp == 1) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank)) SerialSend(digit + '0');
		//else SerialSend(' ');

		temp = temp / 10;
	}
}

//-----------------------------------
void SerialLongVal(uint32_t val)
{
	//unsigned int value;
	uint32_t temp = 1000000000;
	char digit;
	char blank = 1;

	while (temp)
	{
		digit = val / temp;
		val = val % temp;
		temp = temp / 10;
		
		if (!temp) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank))SerialSend(digit + '0');
	}
}
